/*
 * file StpAccessControlledResource.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.StpAccessControlledResource
 *
 * (C) Copyright IBM Corporation 2007, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import java.util.List;

import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

/**
 * The interface implemented by resources that support access control lists
 */
public interface StpAccessControlledResource extends StpResource
{
    /**
     * A Boolean indicating whether or not the session user is allowed to change
     * the access rights on this resource.
     */
    PropertyName<Boolean> CAN_CHANGE_ACCESS_RIGHTS =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "can-change-access-rights");

    /**
     * Returns the value of the
     * {@link #CAN_CHANGE_ACCESS_RIGHTS CAN_CHANGE_ACCESS_RIGHTS} property as
     * defined by this proxy.
     * 
     * @return <b>true</b> if the user has CHANGE_PERMISSION access rights on
     *         this resource; <b>false</b> otherwise.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #CAN_CHANGE_ACCESS_RIGHTS CAN_CHANGE_ACCESS_RIGHTS}
     *             property.
     */
     boolean getCanChangeAccessRights() throws WvcmException;

    /**
     * A list of the access control entries applied directly to this resource.
     */
    PropertyName<List<StpAccessControlEntry>> APPLIED_ACCESS_CONTROL_ENTRIES =
        new PropertyName<List<StpAccessControlEntry>>(PROPERTY_NAMESPACE,
                                                      "applied-access-control-entries");

    /**
     * Returns the value of the
     * {@link #APPLIED_ACCESS_CONTROL_ENTRIES APPLIED_ACCESS_CONTROL_ENTRIES}
     * property as defined by this proxy.
     * 
     * @return A collection of StpAccessControlEntry instances. Will never be
     *         <b>null</b>. The controlled resource of each instance will refer
     *         to this resource, the type of each will be APPLIED, and the
     *         principal and access rights fields will vary based on the access
     *         rights applied to this resource.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #APPLIED_ACCESS_CONTROL_ENTRIES APPLIED_ACCESS_CONTROL_ENTRIES}
     *             property.
     */
    List<StpAccessControlEntry> getAppliedAccessControlEntries()
        throws WvcmException;

    /**
     * A list of the access control entries in effect on this resource for all
     * principals known to the user.
     */
    PropertyName<List<StpAccessControlEntry>> EFFECTIVE_ACCESS_CONTROL_ENTRIES =
        new PropertyName<List<StpAccessControlEntry>>(PROPERTY_NAMESPACE,
                                                      "effective-access-control-entries");

    /**
     * Returns the value of the
     * {@link #EFFECTIVE_ACCESS_CONTROL_ENTRIES EFFECTIVE_ACCESS_CONTROL_ENTRIES}
     * property as defined by this proxy.
     * 
     * @return A collection of StpAccessControlEntry instances. Will never be
     *         <b>null</b>. The controlled resource of each instance will refer
     *         to this resource, the type of each will be EFFECTIVE, and the
     *         principal and access rights fields will vary based on the
     *         relevant access control entries applied to this and related
     *         resources.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #EFFECTIVE_ACCESS_CONTROL_ENTRIES EFFECTIVE_ACCESS_CONTROL_ENTRIES}
     *             property.
     */
    List<StpAccessControlEntry> getEffectiveAccessControlEntries()
        throws WvcmException;

    /**
     * The effective access rights granted the current user to this resource
     */
    PropertyName<StpAccessControlEntry> ACCESS_RIGHTS =
        new PropertyName<StpAccessControlEntry>(PROPERTY_NAMESPACE,
                                                "access-rights");

    /**
     * Returns the value of the {@link #ACCESS_RIGHTS ACCESS_RIGHTS} property as
     * defined by this proxy.
     * 
     * @return An StpAccessControlEntry instance representing the effective
     *         access rights granted the current user to this resource. Will
     *         never be <b>null</b>. The controlled resource will refer to this
     *         folder, the type will be EFFECTIVE, and the access rights field
     *         will specify the rights granted.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #ACCESS_RIGHTS ACCESS_RIGHTS} property.
     */
    StpAccessControlEntry getAccessRights() throws WvcmException;

    /**
     * A list of access control entries each representing an access right that
     * could be applied to this resource. The principal and controlled resource
     * fields of the returned objects are set nominally to the everyone group
     * and this resource respectively.
     */
    PropertyName<List<StpAccessControlEntry>> POSSIBLE_ACCESS_RIGHTS =
        new PropertyName<List<StpAccessControlEntry>>(PROPERTY_NAMESPACE,
                                                      "possible-access-rights");

    /**
     * Returns the value of the
     * {@link #POSSIBLE_ACCESS_RIGHTS POSSIBLE_ACCESS_RIGHTS} property as
     * defined by this proxy.
     * 
     * @return A collection of StpAccessControlEntry instances. Will never be
     *         <b>null</b>. The controlled resource of each instance will refer
     *         to this resource, the type of each will be APPLIED, the principal
     *         of each will be a group including everyone, and the kind will
     *         vary to enumerate the possible access rights that may be applied
     *         to this folder. Any of these instances may be modified to form an
     *         instance for defining new applied access rights.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #POSSIBLE_ACCESS_RIGHTS POSSIBLE_ACCESS_RIGHTS}
     *             property.
     */
    List<StpAccessControlEntry> getPossibleAccessRights() throws WvcmException;

    /**
     * Defines a new value for the
     * {@link #APPLIED_ACCESS_CONTROL_ENTRIES APPLIED_ACCESS_CONTROL_ENTRIES}
     * property of this proxy.
     * <p>
     * Set new access control entries for the resource. The specified access
     * control entries replace all existing entries for "visible" principals
     * already set on the folder. This operation affects only the principals the
     * user can see, so it may only affect a subset of the full set of access
     * control entries on the resource.
     * 
     * @param value A List of StpAccessControlEntry instances specifying the new
     *            value of the access control list. Only the principal and
     *            access rights of each access control entry are significant.
     *            The type, controlled resource and applied resource fields of
     *            these access control entries are ignored (the effective values
     *            being APPLIED and this resource, respectively).
     */
    void setAppliedAccessControlEntries(List<StpAccessControlEntry> value);

    /**
     * Defines modifications to be made to the
     * {@link #APPLIED_ACCESS_CONTROL_ENTRIES APPLIED_ACCESS_CONTROL_ENTRIES}
     * property of the resource referenced by this proxy.
     * 
     * @param additions Access control entries to be added to those already set
     *            on the resource; does not add duplicates, and does not produce
     *            an error if there are duplicates. An entry on this list
     *            replaces any existing entry for the same principal. Does not
     *            add an access control entry for a principal the user is not a
     *            member of, unless the user also has the Security Administrator
     *            privilege.
     * @param deletions All access control entries in this list that match any
     *            visible entries on the resource are removed from the respource;
     *            matching does not consider the access rights or controlled
     *            resource fields.
     */
    void setAppliedAccessControlEntries(List<StpAccessControlEntry> additions,
                                        List<StpAccessControlEntry> deletions);
}
